[![Build Status](https://github.com/lessthanoptimal/BoofCV/actions/workflows/gradle.yml/badge.svg)](https://github.com/lessthanoptimal/BoofCV/actions/workflows/gradle.yml)
[![Join the chat at https://gitter.im/lessthanoptimal/BoofCV](https://badges.gitter.im/Join%20Chat.svg)](https://gitter.im/lessthanoptimal/BoofCV?utm_source=badge&utm_medium=badge&utm_campaign=pr-badge&utm_content=badge)
[![Maven Central](https://img.shields.io/maven-central/v/org.boofcv/boofcv-core.svg)](https://maven-badges.herokuapp.com/maven-central/org.boofcv/boofcv-core)

------------------------------------------------------
# Table of Contents

* [Introduction](#introduction)
  * [Cloning Repository](#cloning-git-repository)
  * [Quick Start](#quick-start-examples-and-demonstrations)
  * [Gradle and Maven](#adding-to-gradle-and-maven-projects)
* [Building from Source](#building-from-source)
* [Dependencies](#dependencies)
* [Help/Contact](#contact)

------------------------------------------------------
## Introduction

BoofCV is an open source real-time computer vision library written entirely in Java and released under the Apache License 2.0. Functionality includes low-level image processing, camera calibration, feature detection/tracking, structure-from-motion, classification, and recognition.

- [ Project Webpage ]( http://boofcv.org                                  )
- [ Message Board   ]( https://groups.google.com/group/boofcv             )
- [ Bug Reports     ]( https://github.com/lessthanoptimal/BoofCV/issues   )
- [ Repository      ]( https://github.com/lessthanoptimal/BoofCV          )

## Cloning GIT Repository

The bleeding edge source code can be obtained by cloning the git repository.

```
git clone -b SNAPSHOT --recursive https://github.com/lessthanoptimal/BoofCV.git boofcv
```

Is the data directory empty? That's because you didn't follow instructions and skipped --recursive. Fix that by doing the following.
```
cd boofcv
git submodule update --init --recursive
```

## Quick Start Examples and Demonstrations
To run BoofCV you need Java 11 or newer and any free version will work, e.g. [Zulu](https://www.azul.com/downloads/?package=jdk). To build BoofCV, just let the Gradle script handle everything. It will automatically download what it needs and ignore your local JDK if it's not compatible.

```bash
cd boofcv
git clean -fd main # Removes stale autogenerated code
./gradlew autogenerate
./gradlew examplesJar
java -jar examples/examples.jar
./gradlew demonstrationsJar
java -jar demonstrations/demonstrations.jar
```

All the code for examples and demonstrations is in boofcv/examples and boofcv/demonstrations. Example code is intended easy to understand, so look there first.

Click [HERE](applications/readme.md) for instruction on building and running Applications.
There you can calibrate cameras, create QR codes, batch scan for QR codes, batch downsample images, ... etc.

## Maven Central Repository

BoofCV is on [Maven Central](http://search.maven.org/) and can be easily added to your favorite build systems; Maven, Gradle, ... etc. It's divided up into many modules, but most people will just need 'boofcv-core'. Additional modules are listed below for GUI and IO components. To include it in a Gradle project add the following to your dependencies:

For Gradle projects:
```
dependencies {
    api(group: 'org.boofcv', name: 'boofcv-core', version: '0.41')
}
```

Here are a list of the most commonly used modules and what they are for:

| Name                 | Description                                                                                                          |
|----------------------|----------------------------------------------------------------------------------------------------------------------|
| boofcv-core          | All the core libraries without any of the integration modules listed below                                           |
| boofcv-android       | Useful functions for working inside of Android devices.                                                              |
| boofcv-ffmpeg        | [javacpp-presets](https://github.com/bytedeco/javacpp-presets) their ffmpeg wrapper is used for reading video files. |
| boofcv-javacv        | [JavaCV](https://github.com/bytedeco/javacv) is a wrapper around OpenCV mainly for file IO.                          |
| boofcv-jcodec        | [JCodec](http://jcodec.org/) is a pure Java video reader/writer.                                                     |
| boofcv-kotlin        | [Kotlin](https://kotlinlang.org/) extensions which take advantage of Kotlin's unique features.                       |
| boofcv-pdf           | Needed to render fiduals as PDF documents                                                                            |
| boofcv-swing         | Visualization using Java Swing. Required for examples and demonstrations.                                            |
| boofcv-WebcamCapture | A few functions that make [WebcamCapture](http://webcam-capture.sarxos.pl/) even easier to use.                      |

## Directories

| Directory       | Description                                                                                                         |
|-----------------|---------------------------------------------------------------------------------------------------------------------|
| applications/   | Helpful applications                                                                                                |
| data/           | Directory containing optional data used by applets and examples.                                                    |
| demonstrations/ | Demonstration code which typically lets experiment by changing parameters in real-time                              |
| examples/       | Set of example code designed to be easy to read and understand.                                                     |
| integration/    | Contains code which allows BoofCV to be easily integrated with 3rd party libraries. Primary for video input/output. |
| main/           | Contains the source code for BoofCV                                                                                 |

------------------------------------
# Building from Source

Building and installing BoofCV into your local Maven repository is easy[1] using the [gradlew](https://docs.gradle.org/current/userguide/gradle_wrapper.html) script:
```bash
cd boofcv
git clean -fd main                # Removes stale autogenerated code
./gradlew autogenerate            # Creates auto generated files
./gradlew publishToMavenLocal     # Installs it into the local maven repository 
```
If you wish to have jars instead, the following commands are provided.
```bash
./gradlew oneJarBin               # Builds a single jar with all of BoofCV in it
./gradlew createLibraryDirectory  # Puts all jars and dependencies into boofcv/library
./gradlew alljavadoc              # Combines all JavaDoc from all sub-projects into a single set
```

[1] A couple of the integration submodules have a custom build process that can't be performed by Gradle. The script is smart enough to ignore modules and tell you that it is doing so if you haven't configured it yet.

## IntelliJ

IntelliJ is the recommended IDE for use with BoofCV. With IntelliJ you can directly import the Gradle project. 

1. File->Project From Existing Sources
2. Select your local "boofcv" directory
3. Confirm that you wish to import the Gradle project

## Eclipse

Similar to IntelliJ, you can now import Gradle projects like BoofCV using the Buildship tooling. [Instructions](https://www.vogella.com/tutorials/EclipseGradle/article.html)

-----------------------------------------------------------
# Dependencies

Core BoofCV modules depends on the following libraries

- [ args4f        ]( http://args4j.kohsuke.org/)
- [ EJML          ]( http://code.google.com/p/efficient-java-matrix-library )
- [ GeoRegression ]( http://georegression.org )
- [ DDogleg       ]( http://ddogleg.org)
- [ DeepBoof      ]( https://github.com/lessthanoptimal/DeepBoof)

The following is required for unit tests

- [ JUnit   ]( http://junit.sourceforge.net/)

Code from the following libraries has been integrated into BoofCV

- [General Purpose FFT by Takuya Ooura](http://www.kurims.kyoto-u.ac.jp/~ooura/fft.html)
  * Java port by Piotr Wendykier with modifications by Peter Abeles to recycle memory.
  
The optional sub-projects in integration also have several dependencies. See those sub-projects for a list of their dependencies.

------------------------------------
# Contact

For questions or comments about BoofCV please use the message board. Only post a bug report after doing some due diligence to make sure it is really a bug and that it has not already been reported.

[Message Board](http://groups.google.com/group/boofcv)
